use actix_web::{web, HttpResponse, Responder};
use serde::Deserialize;
use sqlx::PgPool;

use crate::models::{Assignment, Event, Function};

#[derive(Deserialize)]
pub struct NewAssignmentsForm {
    event: i32,
    wachhabender: Option<i32>,
    posten: Vec<i32>
}

#[actix_web::post("/assignments/new")]
pub async fn post(pool: web::Data<PgPool>, form: web::Form<NewAssignmentsForm>) -> impl Responder {

    let event = Event::read_by_id_including_location(&pool, form.event).await.unwrap(); // TODO: Check if location is needed

    if event.voluntary_wachhabender && form.wachhabender.is_some() && form.posten.contains(&form.wachhabender.unwrap()) {
        return HttpResponse::BadRequest().body("Wachhabender kann nicht zugleich Posten sein!");
    }

    let mut joined_ids = form.posten.clone();
    if let Some(id) = form.wachhabender {
        joined_ids.push(id);
    }

    for availabillity_id in joined_ids {
        let assignments = Assignment::read_by_availabillity(pool.get_ref(), availabillity_id).await.unwrap();

        let mut can_be_used = true;

        for assignment in assignments {
            if event.start_time >= assignment.start_time && event.start_time <= assignment.end_time {
                can_be_used = false;
                break;
            }
        }

        if !can_be_used {
            return HttpResponse::BadRequest().body("availabillity time slot bereits genutzt!");
        }
    }

    if let Some(id) = form.wachhabender {
        Assignment::create(pool.get_ref(), id, event.id, Function::Wachhabender, event.start_time, event.end_time).await.unwrap();
    }

    for id in &form.posten {
        Assignment::create(pool.get_ref(), *id, event.id, Function::Posten, event.start_time, event.end_time).await.unwrap();
    }

    HttpResponse::Ok().finish()
}
